"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EditableRow = void 0;
const react_1 = __importStar(require("react"));
const lodash_1 = require("lodash");
const use_force_update_1 = __importDefault(require("use-force-update"));
const json_server_orm_1 = require("json-server-orm");
const log_1 = require("./common/log");
const env_1 = require("./common/env");
// FIXME Warning: A component is changing an uncontrolled input of type checkbox to be controlled. Input elements should not switch from uncontrolled to controlled (or vice versa). Decide between using a controlled or uncontrolled input element for the lifetime of the component. More info: https://fb.me/react-controlled-components
function EditableRow({ table, item: editItem, onSubmit, onCancel }) {
    const db = env_1.env.db;
    const validation = env_1.env.validation;
    const dbTable = react_1.useMemo(() => db[table], [table]);
    const fields = react_1.useMemo(() => validation.getOwnFields(table), [table]);
    const [item] = react_1.useState(editItem ? lodash_1.cloneDeep(editItem) : validation.getDefaultItem(table));
    const [error, setError] = react_1.useState();
    const handleClick = async () => {
        try {
            log_1.log('[client.EditableRow.handleClick]', item);
            const newItem = editItem ? await dbTable.update(item) : await dbTable.add(item);
            log_1.log('[client.EditableRow.handleClick] newItem', newItem);
            onSubmit && onSubmit(newItem);
        }
        catch (err) {
            if (err.response)
                err = new json_server_orm_1.ValidationError(err.response.data.message, err.response.data.field);
            if (err instanceof json_server_orm_1.ValidationError && err.field)
                setError(err);
            else
                alert(err.message);
        }
    };
    return react_1.default.createElement("tr", { className: 'row-edit' },
        react_1.default.createElement("td", { className: 'pad' },
            react_1.default.createElement("i", { className: 'icon submit', onClick: handleClick }),
            react_1.default.createElement("i", { className: 'icon cancel', onClick: () => onCancel && onCancel() })),
        fields.map(f => (react_1.default.createElement("td", { key: f },
            react_1.default.createElement(EditableField, { table: table, item: item, f: f, error: error })))));
}
exports.EditableRow = EditableRow;
function EditableField({ table, item, f, error }) {
    if (f === 'id')
        return react_1.default.createElement("span", { className: 'pad' }, item[f] || '');
    const validation = env_1.env.validation;
    const ps = react_1.useMemo(() => validation.getOwnProperties(table), [table]);
    const forceUpdate = use_force_update_1.default();
    const handleChange = (ev) => {
        const type = ps[f].type;
        switch (type) {
            case 'number':
            case 'integer':
            case 'string':
            case 'null':
            case 'object':
            case 'array':
                item[f] = ev.target.value;
                break;
            case 'boolean':
                item[f] = ev.target.checked;
                break;
        }
        forceUpdate();
    };
    const handleBlur = (ev) => {
        const f = ev.target.name;
        const type = ps[f].type;
        const rawvalue = ev.target.value;
        let newvalue;
        switch (type) {
            case 'number':
                newvalue = parseFloat(rawvalue);
                break;
            case 'integer':
                newvalue = parseInt(rawvalue);
                break;
            case 'string':
                newvalue = rawvalue;
                break;
            case 'boolean':
                newvalue = ev.target.checked;
                break;
            case 'object':
            case 'array':
                newvalue = JSON.parse(rawvalue);
                break;
            case 'null':
                newvalue = null;
                break;
        }
        item[f] = newvalue;
        forceUpdate();
    };
    const type = ps[f].type;
    const hasError = error?.field === f;
    const errUI = hasError ? react_1.default.createElement("span", { className: "error" }, error?.message) : react_1.default.createElement(react_1.default.Fragment, null);
    const className = 'field type-' + type + (hasError ? ' error' : '');
    switch (type) {
        case 'number':
        case 'integer':
        case 'string':
            return react_1.default.createElement(react_1.default.Fragment, null,
                react_1.default.createElement("input", { name: f, type: 'text', className: className, maxLength: ps[f].maxLength, minLength: ps[f].minLength, value: item[f], onChange: handleChange, onBlur: handleBlur }),
                errUI);
        case 'boolean':
            return react_1.default.createElement("input", { name: f, type: 'checkbox', className: className, checked: item[f], onChange: handleChange, onBlur: handleBlur });
        case 'object':
        case 'array':
            return react_1.default.createElement(react_1.default.Fragment, null,
                react_1.default.createElement("textarea", { name: f, className: className, maxLength: ps[f].maxLength, minLength: ps[f].minLength, value: typeof item[f] === 'string' ? item[f] : JSON.stringify(item[f], null, 2), onChange: handleChange, onBlur: handleBlur }),
                errUI);
        case 'null':
            return react_1.default.createElement("span", { className: 'readonly' }, "null");
            break;
    }
}
